@@ -66,22 +66,27 @@ def decode_type_specifier(encoded):
     return {'kind': 'specifier', 'type': decoded, 'tail': decoded['tail'], 'specifier': TYPE_SPECIFIERS[encoded[0]]}
 
 
-def decode_struct(encoded: str):
-    close_index = index_of_closing_char(encoded, '{', '}')
-    struct_str = encoded[1:close_index]
-    try:
-        name = struct_str[:struct_str.index('=')]
-    except ValueError:
-        name = ''
-    else:
-        struct_str = struct_str[len(name) + 1:]
+def decode_fielded_type(encoded, kind, open, close):
+    close_index = index_of_closing_char(encoded, open, close)
     long_tail = encoded[close_index + 1:]
-    types_in_struct = []
-    while struct_str:
-        decoded = decode_type_recursive(struct_str)
-        types_in_struct.append(decoded)
-        struct_str = decoded['tail']
-    return {'kind': 'struct', 'types': types_in_struct, 'name': name, 'tail': long_tail}
+    type_str = encoded[1:close_index]
+    if '=' not in type_str:
+        return {'kind': kind, 'types': None, 'name': type_str, 'tail': long_tail}
+    name, type_str = type_str.split('=', 1)
+    types_in_str = []
+    while type_str:
+        decoded = decode_type_recursive(type_str)
+        types_in_str.append(decoded)
+        type_str = decoded['tail']
+    return {'kind': kind, 'types': types_in_str, 'name': name, 'tail': long_tail}
+
+
+def decode_struct(encoded: str):
+    return decode_fielded_type(encoded, 'struct', '{', '}')
+
+
+def decode_union(encoded: str):
+    return decode_fielded_type(encoded, 'union', '(', ')')
 
 
 def decode_array(encoded: str):
@@ -122,6 +127,8 @@ def decode_type_recursive(encoded: str):
         return decode_name(encoded)
     elif encoded[0] == 'b':
         return decode_bit_fields(encoded)
+    elif encoded[0] == '(':
+        return decode_union(encoded)
     return decode_name(f'"{encoded}"')
 
 
@@ -150,9 +157,12 @@ def description_for_simple(type_dictionary):
     return type_dictionary['type']
 
 
-def description_for_struct(type_dictionary):
-    name = (type_dictionary['name'] + ' ') if type_dictionary['name'] != '?' else ''
-    desc = 'struct ' + name + '{ '
+def description_for_fielded_type(type_dictionary, type_name):
+    name = type_dictionary['name'] if type_dictionary['name'] != '?' else ''
+    desc = f'{type_name} ' + name
+    if type_dictionary['types'] is None:
+        return desc
+    desc = desc.rstrip(' ') + ' { '
     for i, type_ in enumerate(type_dictionary['types']):
         if type_['kind'] == 'array':
             desc += description_for_type(type_['type']) + f' x{i}[{type_["count"]}]; '
@@ -164,6 +174,14 @@ def description_for_struct(type_dictionary):
     return desc
 
 
+def description_for_struct(type_dictionary):
+    return description_for_fielded_type(type_dictionary, 'struct')
+
+
+def description_for_union(type_dictionary):
+    return description_for_fielded_type(type_dictionary, 'union')
+
+
 def description_for_array(type_dictionary):
     return description_for_type(type_dictionary['type']) + f' x[{type_dictionary["count"]}]'
 
@@ -186,6 +204,7 @@ def description_for_type(type_dictionary):
         'array': description_for_array,
         'name': description_for_name,
         'bitfield': description_for_bitfield,
+        'union': description_for_union,
     }[type_dictionary['kind']](type_dictionary)
 
 
